#include <linux/ioctl.h>
#include <linux/cdev.h>

enum bar0_regs_t {
    HW_FW_REV = 0x0D
};

struct pmc_e1_cas_data
{
    unsigned char chan_num;
    unsigned char cdata[16];
};

struct pmc_e1_controller
{
	struct pci_dev		*pci_device;	//coresponding pci_dev struct
	struct list_head	dev_node;	//list of devices, may be we don't need it now...
	struct cdev		dev_file;	//descriptop file
	spinlock_t              dev_lock;
	unsigned char           dev_number;
	volatile unsigned int	*bar0;
	volatile unsigned int	*bar1;		//pointers to map device's BARS
	unsigned char           e1_amount;      //number of last e1 if
	unsigned char		initialized:1;
	unsigned char		opened:1;
	unsigned char		registered:1;
	char                    name[30];
	unsigned char           pos_in_write_buf;
	unsigned char           have_liu[8];  	/* TODO: define max channels per board, define pmc_port struct ? */
	void *                  priv_data;      //hook for dahdi
	unsigned int            test_sig;
	struct device     	*pmc_class_dev;
	struct dahdi_device 	*ddev;
};

#define getp_pmc_struct(pdev) list_entry(pdev, pmc_e1_controller, pci_dev)

#define PMC_BIT_(a)  (1<<a)

/** Selection of mode & a/b select*/
enum mode1_hp {
	MODE1_DMARST_HP = PMC_BIT_(25),
	MODE1_CT_TI_EN_HP = PMC_BIT_(26),
	MODE1_ABSEL_HP = PMC_BIT_(22),           /**0=use A signals/1=use B signals*/
	MODE1_MASL_HP = PMC_BIT_(23),    /**0=slave on CT-BUS, 1=Master on CT-BUS*/
	MODE1_CLR_ZERO_HP = PMC_BIT_(20),       /**0=do not zero 0 ts, 1=zero 0 timeslot*/
	MODE1_CONS_DEL_HP = PMC_BIT_(15),	/**1==condtant delay through switch table*/
};	

//definitions for quasar-m framer status
#define FRS_FRAME_SYNC   (1 << 23)
#define FRS_MFRAME_SYNC  (1 << 22)
#define FRS_CRC_SYNC     (1 << 21)
#define FRS_FRAME_ERR    (1 << 20)
#define FRS_CRC4_ERR     (1 << 19)
#define FRS_RED          (1 << 18)
#define FRS_AIS          (1 << 17)
#define FRS_MFRAI        (1 << 16)
#define FRS_RAI_RX       (1 << 15)
#define FRS_EBIT_RX1     (1 << 14)
#define FRS_EBIT_RX0     (1 << 13)
#define FRS_SA_BITS4     (1 << 12)
#define FRS_SA_BITS3     (1 << 11)
#define FRS_SA_BITS2     (1 << 10)
#define FRS_SA_BITS1     (1 << 9)
#define FRS_SA_BITS0     (1 << 8)
#define FRS_SLIP_ERR     (1 << 4)
#define FRS_SLIP_DIR     (1 << 3)
#define FRS_LOS          (1 << 2)
#define FRS_HDB3_ERR     (1 << 1)
#define FRS_AMI_ERR       1

#define FRC1_ADDR_OFFSET 0
#define FRS_ADDR_OFFSET 2
#define FRS_HDB3ERR_OFFSET 4
#define FRS_FRAMEERR_OFFSET 5
#define FRS_CRC4ERR_OFFSET 6
#define FRS_EBITERR_OFFSET 7
#define SWITCH_TABLE_OFFSET 0x2000
#define INPUT_TS_SIGNATURE(LINE, TS) ( (LINE << 8) | TS)

//definitions for CAS related registers
#define CAS_BIT_BUFFER_OFFSET 0x10
#define CAS_Y_BIT 	4
#define CAS_X1_BIT 	8
#define CAS_X2_BIT 	2
#define CAS_X3_BIT 	1
#define CAS_RX_Y_BIT 	(4 << 8)
#define CAS_RX_X1_BIT 	(8 << 8)
#define CAS_RX_X2_BIT 	(2 << 8)
#define CAS_RX_X3_BIT 	(1 << 8)



//definitions for framer configuration
#define FRC_ACRC      (1 << 26)
#define FRC_ARAI      (1 << 25)
#define FRC_AEBIT     (1 << 24)
#define FRC_RAI_TX    (1 << 23)
#define FRC_EBIT_TX1  (1 << 22)
#define FRC_EBIT_TX0  (1 << 21)
#define FRC_SA_BITS4  (1 << 20)
#define FRC_SA_BITS3  (1 << 19)
#define FRC_SA_BITS2  (1 << 18)
#define FRC_SA_BITS1  (1 << 17)
#define FRC_SA_BITS0  (1 << 16)
#define FRC_TXAO      (1 << 12)
#define FRC_FR_RX_EN  (1 << 11)
#define FRC_FR_TX_EN  (1 << 10)
#define FRC_CRC4_TX_EN    (1 << 9)
#define FRC_MFR_TX_EN     (1 << 8)
#define FRC_SHORT_HAUL_EN (1 << 7)
#define FRC_TXEN          (1 << 6)
#define FRC_HDB3_RX_EN    (1 << 5)
#define FRC_HDB3_TX_EN    (1 << 4)
#define FRC_MLBK          (1 << 2)
#define FRC_DBLK          (1 << 1)
#define FRC_RLBK           1


//definitions for LIU bitfields
#define LR_CR1_JASEL0     (1 << 6)
#define LR_CR1_UNIEB      (1 << 4)
#define LR_CR1_ENCENB     (1 << 5)
#define LR_CR1_EC1         1
#define LR_CR1_EC4        (1 << 3)
#define LR_CR4_CODEV       1
#define LR_CR4_ZEROV       2

#define LR_CR2_ERLOOP	  (1 << 0)
#define LR_CR2_ELLOOP	  (1 << 1)
#define LR_CR2_EALOOP	  (1 << 2)
#define LR_CR2_ENLOOP	  (1 << 3)
#define LR_CR2_ETAOS      (1 << 4)
#define LR_CR2_EPAT0      (1 << 5)
#define LR_CR2_EPAT1      (1 << 6)
#define LR_CR2_RESET      (1 << 7)

#define LR_PSR_LOS	  (1 << 0)
#define LR_PSR_NLOOP	  (1 << 1)
#define LR_PSR_AIS	  (1 << 2)
#define LR_PSR_QRSS	  (1 << 3)
#define LR_PSR_RESERV4	  (1 << 4)
#define LR_PSR_DFMO	  (1 << 5)
#define LR_PSR_BIST	  (1 << 6)
#define LR_PSR_RESERV7	  (1 << 7)

//LIU quasar
#define LIU_CTRL 0x12
#define LIU_N (1 << 10)
#define LIU_CS (1 << 9)
#define LIU_TR (1 << 8)
#define LIU_WRITE 0
#define LIU_READ 1

//LIU LXT

#define LR_CR1 0
#define LR_CR2 1
#define LR_CR3 2
#define LR_ICR 3
#define LR_TSR 4
#define LR_PSR 5
#define LR_ESR 6
#define LR_CR4 7

int acc_lxt_reg(struct pmc_e1_controller * tmp_dev ,unsigned char chan_num, unsigned char addr, unsigned char mode, unsigned char *data);

#define module_printk(level, fmt, args...) printk(level "%s: " fmt, THIS_MODULE->name, ## args)
#ifdef PMC_E1_DEBUG
# define module_dprintk(level, fmt, args...) \
    module_printk(level, fmt, ## args)
# define dprintk(level, fmt, args...) \
    printk(level fmt, ## args)
#else
# define module_dprintk(level, fmt, args...)
# define dprintk(level, fmt, args...)
#endif
